#!/usr/bin/env python3

import networkx as nx
import csv
import matplotlib.pyplot as plt
import sys

if len(sys.argv) != 2: sys.exit("Required argument: edgelist-CSV-file")
G = nx.DiGraph()

with open(sys.argv[1]) as f:
    reader = csv.reader(f, delimiter = " ", skipinitialspace = True)

    # Skip title lines:
    next(reader) 
    next(reader)
    
    for row in reader:
        k1, i1, k2, i2, weight = int(row[0]), int(row[1]), int(row[2]), \
                                 int(row[3]), float(row[4])
        G.add_edge((k1, i1), (k2, i2), weight = weight)

print("Number of nodes:", len(G))
print("Number of edges:", G.number_of_edges())
n1 = 0
n2 = 0
print("Merging:")

for n, d in G.in_degree():
    if d >= 1:
        n2 += 1
        if d >= 2: print(list(G.predecessors(n)), "->", n)

print("---")
print("Splitting:")

for n, d in G.out_degree():
    if d >= 1:
        n1 += 1
        if d >= 2: print(n, "->", list(G.successors(n)))

print("---")
print("number of nodes with at least one successor =", n1)
print("number of nodes with at least one predecessor =", n2)
pos = nx.nx_agraph.graphviz_layout(G, prog = "dot")
plt.figure()
nx.draw(G, pos, with_labels=True)
plt.show()
