module read_snapshot_m

  implicit none

contains

  subroutine read_snapshot(s, k, hshp, corner, step, nlon, nlat, copy, k1, &
       ishape_last)

    ! Note: read_field_indices must have been called before read_snapshot.

    ! Libraries:
    use contour_531, only: convert_to_ind 
    use nr_util, only: assert

    use derived_types, only: snapshot, eddy, shp_triplet
    use read_eddy_m, only: read_eddy

    type(snapshot), intent(out):: s ! completely defined
    integer, intent(in):: k ! date index
    TYPE(shp_triplet), intent(in):: hshp

    real, intent(in):: corner(:) ! (2) longitude and latitude of the
    ! corner of the whole grid in input NetCDF, in rad

    real, intent(in):: step(:) ! (2) longitude and latitude steps, in rad

    integer, intent(in):: nlon, nlat
    ! size of ssh array in input NetCDF, assuming no repeated point if
    ! the grid is global

    integer, intent(in):: copy, k1, ishape_last(:)

    ! Local:
    integer ishape, ishape_first
    integer k2 ! date index
    integer i ! eddy index
    type(eddy) e

    !---------------------------------------------------------------------

    if (k == k1) then
       ishape_first = 0
    else
       ! {k > k1}
       ishape_first = ishape_last(k - k1) + 1
    end if

    s%number_vis_extr = ishape_last(k - k1 + 1) - ishape_first + 1
    allocate(s%list_vis(s%number_vis_extr))

    do ishape = ishape_first, ishape_last(k - k1 + 1)
       call read_eddy(e, k2, i, hshp, ishape)

       ! Check that all the eddies have the same date index:
       call assert(k2 == k, "read_snapshot: date index")

       s%list_vis(i) = e
    end do

    allocate(s%ind_extr(2, s%number_vis_extr))
    forall (i = 1:s%number_vis_extr) s%ind_extr(:, i) &
         = nint(convert_to_ind(s%list_vis(i)%coord_extr, corner, step))

    s%number_eddies = s%number_vis_extr

    ! Define s%extr_map from s%ind_extr:

    allocate(s%extr_map(1 - copy:nlon + copy, nlat))
    s%extr_map(1:nlon, :) = 0

    ! Since we are not sure that there is no duplicate tuple in
    ! s%ind_extr, we must use a do construct and not a forall:
    do i = 1, s%number_vis_extr
       s%extr_map(s%ind_extr(1, i), s%ind_extr(2, i)) = i
    end do

    call assert(count(s%extr_map(1:nlon, :) /= 0) == s%number_vis_extr, &
         "read_snapshot: one eddy per grid point")

    ! Extend in longitude:
    s%extr_map(1 - copy:0, :) = s%extr_map(nlon + 1 - copy:nlon, :)
    s%extr_map(nlon + 1:nlon + copy, :) = s%extr_map(1:copy, :)

  end subroutine read_snapshot
  
end module read_snapshot_m
